# -*- coding: utf-8 -*-

###########################################################################
#                                                                         #
#  This file is part of QtARMSim.                                         #
#                                                                         #
#  QtARMSim is free software: you can redistribute it and/or modify       #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation; either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful, but    #
#  WITHOUT ANY WARRANTY; without even the implied warranty of             #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      #
#  General Public License for more details.                               #
#                                                                         #
###########################################################################

# References
# http://stackoverflow.com/questions/21564976/how-to-create-a-proxy-model-that-would-flatten-nodes-of-a-qabstractitemmodel-int

#
# To test this module, execute from qtarmsim upper directory the following command:
#
#    python3 -m qtarmsim.widget.memorylcdview
#

from PySide import QtCore, QtGui
from PySide.QtCore import Qt

from .memorymodel import MemoryModel
from .common import InputToHex, getMonoSpacedFont


class MemoryLCDProxyModel(QtGui.QAbstractProxyModel):
    
    @QtCore.Slot(QtCore.QModelIndex, QtCore.QModelIndex)
    def sourceDataChanged(self, topLeft, bottomRight):
        self.dataChanged.emit(self.mapFromSource(topLeft), \
                              self.mapFromSource(bottomRight))

    # InputToHex helper object
    input2hex = InputToHex()

    def __init__(self, parent=None):
        super(MemoryLCDProxyModel, self).__init__(parent)
        # Set font
        id = QtGui.QFontDatabase.addApplicationFont(":/fonts/lcd plus.ttf")
        self.qFont = QtGui.QFont("lcd plus")
        self.qFont.setPointSize(24)
        #self.qFont.setBold(True)
        # Set brush
        self.qBrush = QtGui.QBrush(QtGui.QColor(100, 100, 100, 30), Qt.SolidPattern)
        # 120, 174, 77, 0), Qt.SolidPattern)
        # Initial values of LCDRows, LCDColumns, and memoryBankRow
        self.LCDRows = 0
        self.LCDColumns = 0
        self.memoryBankRow = -1
        
    def setSourceModel(self, model, hexStartAddress, LCDColumns, LCDRows):
        super(MemoryLCDProxyModel, self).setSourceModel(model)
        self.hexStartAddress = hexStartAddress
        self.startAddress = int(self.hexStartAddress, 16)
        self.LCDColumns = LCDColumns
        self.LCDRows = LCDRows
        self.sourceModel().connect(QtCore.SIGNAL("layoutChanged()"), self.layoutChanged)
        self.sourceModel().dataChanged.connect(self.sourceDataChanged)

    @QtCore.Slot()
    def layoutChanged(self):
        """
        It updates the memoryBank information. It should be triggered by a layoutChanged signal from source model.
        """
        (self.memoryBankRow, sourceModelMemoryBank) = self.sourceModel().getMemoryBank(self.hexStartAddress)
        if self.memoryBankRow != -1:
            self.memoryBankItem = self.sourceModel().rootItem.child(self.memoryBankRow)
            self.memoryBankIndex = self.sourceModel().createIndex(self.memoryBankRow, 0, self.memoryBankItem)
            self.memoryBankStartAddress = int(self.memoryBankItem.data(0).split(" ")[1], 16)
            self.dataChanged.emit(self.createIndex(0,0), self.createIndex(self.LCDRows-1, self.LCDColumns-1))

    def mapFromSource(self, index):
        # If index is root or a memory bank item, return QModelIndex() 
        if index == QtCore.QModelIndex() or index.parent() == QtCore.QModelIndex or self.memoryBankRow == -1:
            return QtCore.QModelIndex()
        # At this point, index should point to a memory address
        memoryBankRow = index.parent().row()
        if memoryBankRow != self.memoryBankRow:
            return QtCore.QModelIndex()
        # At this point, index should point to a memory address of the memory bank specified at setSourceModel()
        memoryAddress = int(index.internalPointer().data(0), 16)
        row = int((memoryAddress - self.startAddress)/self.LCDColumns)
        column = (memoryAddress - self.startAddress) % self.LCDColumns
        if 0 <= row < self.LCDRows and 0 <= column < self.LCDColumns:
            return self.createIndex(row, column)
        else:
            return QtCore.QModelIndex()

    def mapToSource(self, index):
        if not index.isValid() or self.memoryBankRow == -1:
            return QtCore.QModelIndex()
        memoryRow = index.row()*self.LCDColumns + index.column() + (self.startAddress-self.memoryBankStartAddress)
        return self.sourceModel().index(memoryRow, 0, self.memoryBankIndex)

    def rowCount(self, parent):
        return self.LCDRows

    def columnCount(self, parent):
        return self.LCDColumns

    def index(self, row, column, parent):
        if parent.isValid():
            return QtCore.QModelIndex()
        return self.createIndex(row, column)

    def parent(self, index):
        return QtCore.QModelIndex()

    def _chr(self, hexByte):
        n = int(hexByte, 16)
        if n >= 32 and n <= 126:
            return chr(n)
        else:
            return '·'
        
    def data(self, index, role=Qt.DisplayRole):
        if role == Qt.DisplayRole:
            if self.memoryBankRow != -1 and self.mapToSource(index).isValid():
                return self._chr(self.mapToSource(index).internalPointer().data(1)[2:])
            else:
                return " "
        elif role == Qt.BackgroundRole:
            return self.qBrush
        elif role == Qt.FontRole:
            return self.qFont
        elif role == Qt.TextAlignmentRole:
            return Qt.AlignCenter
        else:
            return None
         
    def headerData(self, section, orientation, role):
        return None
 
    def flags(self, index):
        if not index.isValid():
            return False
        return Qt.ItemIsEnabled



# -*- coding: utf-8 -*-

###########################################################################
#                                                                         #
#  This file is part of QtARMSim.                                         #
#                                                                         #
#  QtARMSim is free software: you can redistribute it and/or modify       #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation; either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful, but    #
#  WITHOUT ANY WARRANTY; without even the implied warranty of             #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      #
#  General Public License for more details.                               #
#                                                                         #
###########################################################################

import sys

from PySide import QtGui, QtCore
from PySide.QtCore import Qt
from qtarmsim.widget.myqtreeview import MyQTreeView

from . memorymodel import MemoryModel
from . common import InputToHex, getMonoSpacedFont


class MemoryByWordProxyModel(QtGui.QSortFilterProxyModel):
    
    # InputToHex helper object
    input2hex = InputToHex()

    @QtCore.Slot(QtCore.QModelIndex, QtCore.QModelIndex)
    def sourceDataChanged(self, topLeft, bottomRight):
        self.dataChanged.emit(self.mapFromSource(topLeft), \
                              self.mapFromSource(bottomRight))

    def __init__(self, parent=None):
        super(MemoryByWordProxyModel, self).__init__(parent)
        # Set fonts
        self.qFont = getMonoSpacedFont()
        self.qFontLast = getMonoSpacedFont()
        self.qFontLast.setWeight(QtGui.QFont.Black)
        # Set brushes
        self.qBrushPrevious = QtGui.QBrush(QtGui.QColor(192, 192, 255, 60), Qt.SolidPattern)
        self.qBrushLast = QtGui.QBrush(QtGui.QColor(192, 192, 255, 100), Qt.SolidPattern) 

    def setSourceModel(self, model):
        super(MemoryByWordProxyModel, self).setSourceModel(model)
        self.sourceModel().dataChanged.connect(self.sourceDataChanged)

    def filterAcceptsRow(self, sourceRow, sourceParent):
        try:
            firstColumnData = sourceParent.internalPointer().child(sourceRow).data(0)
        except AttributeError:
            return True
        if  firstColumnData[0:2] != "0x":
            return True
        elif firstColumnData[-1:] in ('0', '4', '8', 'C', 'c'):
            return True
        else:
            return False
         
    def data(self, index, role=Qt.DisplayRole):
        if not index.isValid():
            return None
        sourceModelIndex = self.mapToSource(index)
        item = sourceModelIndex.internalPointer()
        byteMemoryBankRow = sourceModelIndex.parent().row()
        byteRow = sourceModelIndex.row()
        # Memory bank
        if item.parent() == self.sourceModel().rootItem:
            if role == Qt.DisplayRole:
                return item.data(index.column())
            elif role == Qt.FontRole:
                return self.qFont
            return None
        # Memory address
        if role == Qt.DisplayRole:
            if index.column() == 0:
                return item.data(0)
            else:
                hexBytes = []
                byteMemoryBank = sourceModelIndex.parent().internalPointer()
                byteRow = sourceModelIndex.row()
                for i in range(3,-1,-1):
                    hexBytes.append(byteMemoryBank.child(byteRow + i).data(1)[2:])
                return '0x' + ''.join(hexBytes)
        elif role == Qt.BackgroundRole:
            # If any of the bytes is in modifiedBytes, return qBrushLast
            for i in range(4):
                if (byteMemoryBankRow, byteRow+i) in self.sourceModel().modifiedBytes:
                    return self.qBrushLast
            # If not, if any of the bytes is in previouslyModifiedBytes, return qBrushPrevious
            for i in range(4):
                if (byteMemoryBankRow, byteRow+i) in self.sourceModel().previouslyModifiedBytes:
                    return self.qBrushPrevious
            # If not, return None
            return None
        elif role == Qt.FontRole:
            # If any of the bytes is in modifiedBytes, return qFontLast
            for i in range(4):
                if (byteMemoryBankRow, byteRow+i) in self.sourceModel().modifiedBytes:
                    return self.qFontLast
            else:
                return self.qFont
        else:
            return None

    def flags(self, index):
        if not index.isValid():
            return False
        sourceModelIndex = self.mapToSource(index)
        item = sourceModelIndex.internalPointer()
        if item.parent() == self.sourceModel().rootItem:
            return Qt.ItemIsEnabled
        if  index.column() == 0:
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
        if item.parent().data(0)[:3] == 'ROM':
            return Qt.ItemIsEnabled | Qt.ItemIsSelectable
        return Qt.ItemIsEnabled | Qt.ItemIsSelectable | Qt.ItemIsEditable

    def setData(self, index, value, role = Qt.EditRole):
        (hexValue, err_msg) = self.input2hex.convert(value)
        if not hexValue:
            if err_msg:
                QtGui.QMessageBox.warning(None, self.trUtf8("Input error"), err_msg)
            return False
        sourceModelIndex = self.mapToSource(index)
        item = sourceModelIndex.internalPointer()
        hexAddress = item.data(0)
        self.sourceModel().setWord(hexAddress, hexValue, True)
        return True


if __name__ == "__main__":
    #
    # To test this module, execute from qtarmsim upper directory the following command:
    #
    #    python3 -m qtarmsim.model.memorybywordproxymodel
    #
    app = QtGui.QApplication(sys.argv)
    mainWindow = QtGui.QMainWindow()
    # Memory model
    memoryModel = MemoryModel(app)
    memoryModel.appendMemoryBank('ROM', '0x10010000', ['0x{:02X}'.format(i) for i in range(24)])
    memoryModel.appendMemoryBank('RAM', '0x20020000', ['0x{:02X}'.format(i) for i in range(256)])
    # filterProxy
    filterProxy = MemoryByWordProxyModel(app)
    filterProxy.setSourceModel(memoryModel)
    # treeViewMemory
    treeViewMemory = QtGui.QTreeView()
    treeViewMemory.setModel(filterProxy)
    treeViewMemory.expandAll()
    treeViewMemory.resizeColumnToContents(0)
    treeViewMemory.resizeColumnToContents(1)
    # modify memoryModel data after setting the proxy and the view
    memoryModel.appendMemoryBank('RAM', '0x40040000', ['0x{:02X}'.format(i) for i in range(256)])
    memoryModel.setWord('0x10010000', '0xCCDDEEFF')
    memoryModel.stepHistory()
    memoryModel.setWord('0x10010004', '0xCCDDEEFF')
    # Show main window and enter main loop
    mainWindow.setCentralWidget(treeViewMemory)
    mainWindow.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

###########################################################################
#                                                                         #
#  This file is part of QtARMSim.                                         #
#                                                                         #
#  QtARMSim is free software: you can redistribute it and/or modify       #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation; either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful, but    #
#  WITHOUT ANY WARRANTY; without even the implied warranty of             #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      #
#  General Public License for more details.                               #
#                                                                         #
###########################################################################

import sys

from PySide import QtCore, QtGui
from PySide.QtCore import Qt

from .. model.memorymodel import MemoryModel
from .. model.memorylcdproxymodel import MemoryLCDProxyModel



class MemoryLCDView(QtGui.QTableView):
    
    def __init__(self, parent=None):
        super(MemoryLCDView, self).__init__(parent)
        self.setGridStyle(Qt.NoPen)
        self.horizontalHeader().hide()
        self.verticalHeader().hide()
        self.setFocusPolicy(Qt.NoFocus)
        # #78AE4D
        #                 padding: 8 -8 -8 8;
        #                 padding: 4 -3 -3 4;
        self.setStyleSheet("""
            QTableView { background: transparent;
                         border-width: 18 18 18 18;
                         padding: 4 -4 -4 4;
                         border-image: url(:/images/lcd.png) 18 18 18 18;}
            QTableView::item:hover {background: none;}
        """)
        self.verticalScrollBar().setDisabled(True)
        self.horizontalScrollBar().setDisabled(True)
        self.setFrameStyle(QtGui.QFrame.NoFrame)
                           
        
    def setModel(self, memoryModel, hexStartAddress, LCDColumns = 32, LCDRows = 6):
        self.memoryLCDProxyModel = MemoryLCDProxyModel()
        self.memoryLCDProxyModel.setSourceModel(memoryModel, hexStartAddress, LCDColumns, LCDRows)
        super(MemoryLCDView, self).setModel(self.memoryLCDProxyModel)
        #QtGui.QTableView.setModel(self, self.memoryLCDProxyModel)
        self.LCDColumns = LCDColumns
        self.LCDRows = LCDRows
        #for i in range(self.LCDRows):
        #    self.resizeRowToContents(i)
        #for j in range(self.LCDColumns):
        #    self.resizeColumnToContents(j)
        self.resizeColumnsToContents()
        self.resizeRowsToContents()
        for i in range(self.LCDColumns):
            self.setColumnWidth(i, self.columnWidth(i)-8)
        self.setFixedWidth(18 + 18 + 8 + sum([self.columnWidth(i) for i in range(self.LCDColumns)]))
        self.setFixedHeight(18 + 18 + 8 + sum([self.rowHeight(i) for i in range(self.LCDRows)]))
        #self.setCursor(Qt.OpenHandCursor)

if __name__ == "__main__":
    #
    # To test this module, execute from qtarmsim upper directory the following command:
    #
    #    python3 -m qtarmsim.widget.memorylcdview
    #
    app = QtGui.QApplication(sys.argv)
    mainWindow = QtGui.QMainWindow()
    mainWindow.setGeometry(200, 200, 1000, 400)
    # Memory model
    memoryModel = MemoryModel(app)
    memoryModel.appendMemoryBank('ROM', '0x10010000', ['0x{:02X}'.format(i) for i in range(24)])
    memoryModel.appendMemoryBank('RAM', '0x20070000', ['0x{:02X}'.format(i) for i in range(65, 256)])
    # Memory LCD Wiew
    memoryLCDView = MemoryLCDView(mainWindow)
    memoryLCDView.setModel(memoryModel, '0x20070000', 32, 6)
    # Show main window and enter main loop
    mainWindow.setCentralWidget(memoryLCDView)
    mainWindow.show()
    sys.exit(app.exec_())



# -*- coding: utf-8 -*-

###########################################################################
#                                                                         #
#  This file is part of QtARMSim.                                         #
#                                                                         #
#  QtARMSim is free software: you can redistribute it and/or modify       #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation; either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful, but    #
#  WITHOUT ANY WARRANTY; without even the implied warranty of             #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      #
#  General Public License for more details.                               #
#                                                                         #
###########################################################################

from PySide import QtGui, QtCore
from PySide.QtCore import Qt

from . simpletreemodel import TreeModel, TreeItem
from . common import InputToHex, getMonoSpacedFont
from qtarmsim.widget.myqtreeview import MyQTreeView

import sys

class MemoryBank():
    
    def __init__(self, memType, start, nBytes):
        """Initializes a memory bank instance.
        
        @param memType: The memory type, one of RAM or ROM.
        @param start: The starting address in hexadecimal.
        @param nBytes: The number of bytes.
        """
        self.memType = memType
        self.start = int(start, 16)
        self.length = nBytes
        # Round memory size to a word boundary
        if (nBytes % 4):
            self.length += (4 - nBytes % 4)
        self.end = self.start + self.length - 1

    def addressToRow(self, hexAddress):
        "Given an hexadecimal hexAddress, returns the corresponding row"
        intAddress = int(hexAddress, 16)
        return (intAddress - self.start)


class MemoryModel(TreeModel):
    """
    Tree model that manages the memory banks of a given simulation and their contents.
    """

    memoryBanks = []

    modifiedBytes = []
    previouslyModifiedBytes = []

    # memoryEdited signal, parameters are hex address and hex value
    memoryEdited = QtCore.Signal('QString', 'QString')

    def __init__(self, parent=None):
        """
        Initializes the memory model.
        """
        super(MemoryModel, self).__init__(parent)
        self.rootItem = TreeItem(("Address", "Value"))
        # Set fonts
        self.qFont = getMonoSpacedFont()
        self.qFontLast = getMonoSpacedFont()
        self.qFontLast.setWeight(QtGui.QFont.Black)
        # Set brushes
        self.qBrushPrevious = QtGui.QBrush(QtGui.QColor(192, 192, 255, 60), Qt.SolidPattern)
        self.qBrushLast = QtGui.QBrush(QtGui.QColor(192, 192, 255, 100), Qt.SolidPattern)

    def appendMemoryBank(self, memType, hex_start, membytes):
        """
        Adds a new memory bank to the memory model and populates it.
        """
        self.emit(QtCore.SIGNAL("layoutAboutToBeChanged()"))
        self.memoryBanks.append(MemoryBank(memType, hex_start, len(membytes)))
        memory_bank_item = TreeItem(("{} {}".format(memType, hex_start), ""), self.rootItem)
        self.rootItem.appendChild(memory_bank_item)
        address = self.memoryBanks[-1].start
        # Round memory bytes to a word boundary
        if len(membytes) % 4 != 0:
            membytes += ['0x00']*(4 - len(membytes) % 4)
        # Add the bytes to the model
        for hexByte in membytes:
            hexAddress = "0x{0:0{1}X}".format(address, 8)
            memory_item = TreeItem([hexAddress, hexByte], memory_bank_item)
            memory_bank_item.appendChild(memory_item)
            address += 1
        self.emit(QtCore.SIGNAL("layoutChanged()"))

    def getMemoryBank(self, hexAddress):
        """
        Returns the memory bank that holds the given memory address.
        """
        intAddress = int(hexAddress, 16)
        mbRow = 0
        for memoryBank in self.memoryBanks:
            if memoryBank.start <= intAddress <= memoryBank.end:
                return (mbRow, memoryBank)
            mbRow += 1
        return (-1, None)
    
    def getIndex(self, hexAddress):
        """
        Returns the model index that references the given memory address.
        """
        (mbRow, memoryBank) = self.getMemoryBank(hexAddress)
        memoryRow = memoryBank.addressToRow(hexAddress)
        return self.createIndex(memoryRow, 0, self.rootItem.child(mbRow).child(memoryRow))

    def setByte(self, hexAddress, hexByte, emitMemoryEdited = False):
        """
        Stores the given byte at the given address.
        """
        (mbRow, memoryBank) = self.getMemoryBank(hexAddress)
        memoryRow = memoryBank.addressToRow(hexAddress)
        memoryItem = self.rootItem.child(mbRow).child(memoryRow)
        memoryItem.setData(1, hexByte)
        self.modifiedBytes.append((mbRow, memoryRow))
        self.dataChanged.emit(self.createIndex(memoryRow, 0, self.rootItem.child(mbRow)),
                              self.createIndex(memoryRow, 1, self.rootItem.child(mbRow)))
        if emitMemoryEdited:
            self.memoryEdited.emit(hexAddress, hexByte)

    def setWord(self, hexAddress, hexWord, emitMemoryEdited = False):
        """
        Stores the given word (4 bytes) at the given address (following Little Endian memory organization).
        """
        (mbRow, memoryBank) = self.getMemoryBank(hexAddress)
        memoryRowStart = memoryBank.addressToRow(hexAddress)
        for i in range(4):
            hexByte = "0x{}".format(hexWord[2+i*2:4+i*2])
            memoryRow = memoryRowStart + 3 - i # 3-i due to Little Endian
            memoryItem = self.rootItem.child(mbRow).child(memoryRow)
            memoryItem.setData(1, hexByte)
            self.modifiedBytes.append((mbRow, memoryRow))
        self.dataChanged.emit(self.createIndex(memoryRow    , 0, self.rootItem.child(mbRow)),
                              self.createIndex(memoryRow + 3, 1, self.rootItem.child(mbRow)))
        if emitMemoryEdited:
            self.memoryEdited.emit(hexAddress, hexWord)

    def getWord(self, hexAddress):
        """
        Returns the word (4 bytes) at the given address (following Little Endian memory organization).
        """
        (mbRow, memoryBank) = self.getMemoryBank(hexAddress)
        memoryRowStart = memoryBank.addressToRow(hexAddress)
        hexWord = "0x"
        for i in range(4):
            memoryRow = memoryRowStart + 3 - i # 3-i due to Little Endian
            memoryItem = self.rootItem.child(mbRow).child(memoryRow)
            hexWord += memoryItem.data(1)[2:]
    
    def reset(self):
        """
        Resets the model to its original state in any attached views.
        """
        self.beginResetModel()
        self.memoryBanks.clear()
        self.rootItem.childItems.clear()
        self.clearHistory()
        self.endResetModel()

    def clearHistory(self):
        """
        Clears the history of previously modified bytes.
        """
        self.previouslyModifiedBytes.clear()
        self.modifiedBytes.clear()

    def stepHistory(self):
        """
        Steps one the history of previously modified bytes.
        """
        copyOfPrevious = self.previouslyModifiedBytes[:]
        self.previouslyModifiedBytes = self.modifiedBytes[:]
        self.modifiedBytes.clear()
        for (mbRow, memoryRow) in copyOfPrevious + self.previouslyModifiedBytes:
            self.dataChanged.emit(self.createIndex(memoryRow, 0, self.rootItem.child(mbRow)),
                                  self.createIndex(memoryRow, 1, self.rootItem.child(mbRow)))



# -*- coding: utf-8 -*-

###########################################################################
#                                                                         #
#  This file is part of QtARMSim.                                         #
#                                                                         #
#  QtARMSim is free software: you can redistribute it and/or modify       #
#  it under the terms of the GNU General Public License as published by   #
#  the Free Software Foundation; either version 3 of the License, or      #
#  (at your option) any later version.                                    #
#                                                                         #
#  This program is distributed in the hope that it will be useful, but    #
#  WITHOUT ANY WARRANTY; without even the implied warranty of             #
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      #
#  General Public License for more details.                               #
#                                                                         #
###########################################################################

# References
# http://stackoverflow.com/questions/21564976/how-to-create-a-proxy-model-that-would-flatten-nodes-of-a-qabstractitemmodel-int

import sys

from PySide import QtCore, QtGui
from PySide.QtCore import Qt

from .memorymodel import MemoryModel

from . common import InputToHex, getMonoSpacedFont


class MemoryDumpProxyModel(QtGui.QAbstractProxyModel):
    
    @QtCore.Slot(QtCore.QModelIndex, QtCore.QModelIndex)
    def sourceDataChanged(self, topLeft, bottomRight):
        self.dataChanged.emit(self.mapFromSource(topLeft), \
                              self.mapFromSource(bottomRight))

    # InputToHex helper object
    input2hex = InputToHex()

    def __init__(self, parent=None):
        super(MemoryDumpProxyModel, self).__init__(parent)
        # Set fonts
        self.qFont = getMonoSpacedFont()
        self.qFontLast = getMonoSpacedFont()
        self.qFontLast.setWeight(QtGui.QFont.Black)
        # Set brushes
        self.qBrushPrevious = QtGui.QBrush(QtGui.QColor(192, 192, 255, 60), Qt.SolidPattern)
        self.qBrushLast = QtGui.QBrush(QtGui.QColor(192, 192, 255, 100), Qt.SolidPattern)

    def setSourceModel(self, model, memoryBankRow=0):
        super(MemoryDumpProxyModel, self).setSourceModel(model)
        self.memoryBankRow = memoryBankRow
        self.memoryBankItem = self.sourceModel().rootItem.child(memoryBankRow)
        self.memoryBankIndex = self.sourceModel().createIndex(memoryBankRow, 0, self.memoryBankItem)
        self.memoryBankStartAddress = int(self.memoryBankItem.data(0).split(" ")[1], 16)
        self.sourceModel().dataChanged.connect(self.sourceDataChanged)

    def mapFromSource(self, index):
        # If index is root or a memory bank item, return QModelIndex() 
        if index == QtCore.QModelIndex() or index.parent() == QtCore.QModelIndex:
            return QtCore.QModelIndex()
        # At this point, index should point to a memory address
        memoryBankRow = index.parent().row()
        if memoryBankRow != self.memoryBankRow:
            return QtCore.QModelIndex()
        # At this point, index should point to a memory address of the memory bank specified at setSourceModel()
        memoryAddress = int(index.internalPointer().data(0), 16)
        row = int((memoryAddress - self.memoryBankStartAddress)/16)
        column = (memoryAddress - self.memoryBankStartAddress) % 16
        return self.createIndex(row, column)

    def mapToSource(self, index):
        if not index.isValid() or index.column() == 16:
            return QtCore.QModelIndex()
        memoryRow = index.row()*16 + index.column()
        return self.sourceModel().index(memoryRow, 0, self.memoryBankIndex)

    def rowCount(self, parent):
        if not parent.isValid():
            return int(self.memoryBankItem.childCount()/16)
        else:
            return 0

    def columnCount(self, parent):
        # bytes*16 | ASCII 
        return 17

    def index(self, row, column, parent):
        if parent.isValid():
            return QtCore.QModelIndex()
        return self.createIndex(row, column)

    def parent(self, index):
        return QtCore.QModelIndex()

    def _chr(self, hexByte):
        n = int(hexByte, 16)
        if n >= 32 and n <= 126:
            return chr(n)
        else:
            return '·'
        
    def data(self, index, role=Qt.DisplayRole):
        if role == Qt.DisplayRole:
            if index.column() < 16:
                return self.mapToSource(index).internalPointer().data(1)[2:]
            else:
                chars = []
                byteRow = index.row()*16
                for i in range(16):
                    try:
                        item = self.memoryBankItem.child(byteRow + i)
                    except KeyError:
                        break
                    chars.append(self._chr(item.data(1)[2:]))
                return ''.join(chars)
        elif role == Qt.BackgroundRole:
            byteRow = index.row()*16 + index.column()
            byteMemoryBankRow = self.memoryBankRow
            # If the current byte is in modifiedBytes, return qBrushLast
            if (byteMemoryBankRow, byteRow) in self.sourceModel().modifiedBytes:
                return self.qBrushLast
            # If not, if the bytes is in previouslyModifiedBytes, return qBrushPrevious
            if (byteMemoryBankRow, byteRow) in self.sourceModel().previouslyModifiedBytes:
                return self.qBrushPrevious
            # If not, return None
            return None
        elif role == Qt.FontRole:
            byteRow = index.row()*16 + index.column()
            byteMemoryBankRow = self.memoryBankRow
            if (byteMemoryBankRow, byteRow) in self.sourceModel().modifiedBytes:
                return self.qFontLast
            else:
                return self.qFont
        else:
            return None
         
    def headerData(self, section, orientation, role):
        if role == Qt.DisplayRole:
            if orientation == Qt.Horizontal: 
                if section < 16:
                    return "{:X}".format(section)
                else:
                    return "ASCII"
            elif orientation == Qt.Vertical:
                byteRow = section*16
                return self.memoryBankItem.child(byteRow).data(0)
 
    def flags(self, index):
        if not index.isValid():
            return False
        # Column 16 can not be changed
        if index.column() == 16:
            return Qt.ItemIsEnabled
        # If we are in columns 1 to 16, check if the bank memory is ROM or RAM
        if self.memoryBankItem.data(0)[:3] == 'ROM':
            return Qt.ItemIsEnabled
        else:
            return Qt.ItemIsEditable | Qt.ItemIsEnabled

    def setData(self, index, value, role = Qt.EditRole):
        (hexValue, errMsg) = self.input2hex.convert(value, 8)
        if not hexValue:
            if errMsg:
                QtGui.QMessageBox.warning(None, self.trUtf8("Input error"), errMsg)
            return False
        item = self.mapToSource(index).internalPointer()
        hexAddress = item.data(0)
        self.sourceModel().setByte(hexAddress, hexValue, True)
        return True

if __name__ == "__main__":
    #
    # To test this module, execute from qtarmsim upper directory the following command:
    #
    #    python3 -m qtarmsim.model.memorydumpproxymodel
    #
    app = QtGui.QApplication(sys.argv)
    mainWindow = QtGui.QMainWindow()
    mainWindow.setGeometry(200, 200, 1000, 400)
    # Memory model
    memoryModel = MemoryModel(app)
    memoryModel.appendMemoryBank('ROM', '0x10010000', ['0x{:02X}'.format(i) for i in range(24)])
    memoryModel.appendMemoryBank('RAM', '0x20020000', ['0x{:02X}'.format(i) for i in range(256)])
    # Memory dump proxy model
    memoryDumpProxyModel = MemoryDumpProxyModel()
    memoryDumpProxyModel.setSourceModel(memoryModel, 1)
    # Memory dump view
    memoryDumpView = QtGui.QTableView()
    memoryDumpView.setModel(memoryDumpProxyModel)
    memoryDumpView.resizeColumnsToContents()
    memoryDumpView.resizeRowsToContents()
    # Modify the memoryModel data after setting the proxy and the view
    memoryModel.appendMemoryBank('RAM', '0x40040000', ['0x{:02X}'.format(i) for i in range(256)])
    memoryModel.setByte('0x20020000', '0xCC')
    memoryModel.stepHistory()
    memoryModel.setByte('0x20020002', '0xDD')
    # Show main window and enter main loop
    mainWindow.setCentralWidget(memoryDumpView)
    mainWindow.show()
    sys.exit(app.exec_())
